@@ -28,10 +28,7 @@
 #define DISPLAY_DEBUG(m)	(((m) & I_D_DEBUG)         == I_D_DEBUG)
 #define DISPLAY_UNIT(m)		(((m) & I_D_UNIT)	   == I_D_UNIT)
 
-/* Preallocation constants */
-#define NR_CIFS_PREALLOC	2
-
-struct cifs_stats {
+struct cifs_st {
 	unsigned long long rd_bytes     __attribute__ ((aligned (8)));
 	unsigned long long wr_bytes     __attribute__ ((packed));
 	unsigned long long rd_ops       __attribute__ ((packed));
@@ -41,14 +38,15 @@ struct cifs_stats {
 	unsigned long long fdeletes     __attribute__ ((packed));
 };
 
-#define CIFS_STATS_SIZE	(sizeof(struct cifs_stats))
+#define CIFS_ST_SIZE	(sizeof(struct cifs_st))
 
-struct io_hdr_stats {
-	unsigned int active		__attribute__ ((aligned (4)));
-	unsigned int used		__attribute__ ((packed));
+struct io_cifs {
 	char name[MAX_NAME_LEN];
+	int exist;
+	struct cifs_st *cifs_stats[2];
+	struct io_cifs *next;
 };
 
-#define IO_HDR_STATS_SIZE	(sizeof(struct io_hdr_stats))
+#define IO_CIFS_SIZE	(sizeof(struct io_cifs))
 
 #endif  /* _CIFSIOSTAT_H */
