@@ -28,6 +28,8 @@
 #define __closedir(m)		close_list(m)
 #define __realpath(m,n)		get_realname(m,n)
 #define __gettimeofday(m,n)	get_day_time(m)
+#define __getpwuid(m)		get_usrname(m)
+#define __fork(m)		get_known_pid(m)
 
 #define ROOTDIR		"./tests/root"
 #define ROOTFILE	"root"
@@ -51,10 +53,11 @@
 #define __closedir(m)		closedir(m)
 #define __realpath(m,n)		realpath(m,n)
 #define __gettimeofday(m,n)	gettimeofday(m,n)
+#define __getpwuid(m)		getpwuid(m)
+#define __fork(m)		fork(m)
 
 #endif
 
-
 /*
  ***************************************************************************
  * Functions prototypes
@@ -69,12 +72,16 @@ char *get_env_value
 	(const char *);
 int get_fs_stat
 	(char *, struct statvfs *);
+pid_t get_known_pid
+	(void);
 char *get_realname
 	(char *, char *);
 void get_uname
 	(struct utsname *);
 void get_unix_time
 	(time_t *);
+struct passwd *get_usrname
+	(uid_t);
 void next_time_step
 	(void);
 DIR *open_list
